@@ -3,6 +3,7 @@ import asyncio
 import contextlib
 import functools
 import inspect
+import operator
 import time
 import uuid
 from contextvars import (
@@ -13,6 +14,7 @@ from typing import (
     Callable,
     List,
     NamedTuple,
+    Tuple,
 )
 
 # Constants
@@ -20,6 +22,7 @@ CHAR_SPACE = chr(0x20)
 CHAR_INFO = chr(0x1F6C8) + CHAR_SPACE
 CHAR_CHECK_MARK = chr(0X2713)
 CHAR_BROKEN_BAR = chr(0xA6)
+CHAR_SUPERSCRIPT_ONE = chr(0x00B9)
 
 # Containers
 Frame = NamedTuple('Frame', [
@@ -28,12 +31,32 @@ Frame = NamedTuple('Frame', [
     ('level', int),
     ('timestamp', float),
 ])
+EventLoopTimerSnapshot = NamedTuple('EventLoopTimerSnapshot', [
+    ('block_duration_ratio', float),
+    ('expected_tick_duration', float),
+    ('real_tick_duration', float),
+    ('timestamp', float),
+    ('was_blocked', bool),
+])
 
 # Contextvars
 DO_TRACE: ContextVar[bool] = ContextVar('DO_TRACE', default=True)
 LEVEL: ContextVar[int] = ContextVar('LEVEL', default=0)
 STACK: ContextVar[List[Frame]] = ContextVar('STACK', default=[])
 
+EVENT_LOOP_TIMER_SNAPSHOTS: ContextVar[List[EventLoopTimerSnapshot]] = \
+    ContextVar('EVENT_LOOP_TIMER_SNAPSHOTS', default=[])
+
+
+def divide(
+    *,
+    numerator: float,
+    denominator: float,
+    on_zero_denominator: float,
+) -> float:
+    return \
+        numerator / denominator if denominator > 0.0 else on_zero_denominator
+
 
 def get_function_id(function: Callable, function_name: str = '') -> str:
     # Adding decorators to a function modify its metadata
@@ -65,6 +88,49 @@ def increase_counter(contextvar):
         contextvar.reset(token)
 
 
+def measure_event_loop_skew(clock_id: int):
+
+    def callback_handler(
+        expected_tick_duration: float,
+        start_timestamp: float,
+    ):
+        tolerance: float = 1.05
+        timestamp: float = time.clock_gettime(clock_id)
+        real_tick_duration: float = timestamp - start_timestamp
+        block_duration_ratio: float = divide(
+            numerator=real_tick_duration,
+            denominator=expected_tick_duration,
+            on_zero_denominator=1.0,
+        )
+
+        EVENT_LOOP_TIMER_SNAPSHOTS.get().append(EventLoopTimerSnapshot(
+            block_duration_ratio=block_duration_ratio,
+            expected_tick_duration=expected_tick_duration,
+            real_tick_duration=real_tick_duration,
+            timestamp=start_timestamp,
+            was_blocked=block_duration_ratio >= tolerance,
+        ))
+
+        if LEVEL.get() == 1:
+            schedule_callback(expected_tick_duration)
+
+    def schedule_callback(expected_tick_duration: float):
+        with contextlib.suppress(RuntimeError):
+            callback_handler_args: Tuple[float, float] = (
+                expected_tick_duration, time.clock_gettime(clock_id),
+            )
+
+            loop = asyncio.get_running_loop()
+            loop.call_later(
+                expected_tick_duration,
+                callback_handler,
+                *callback_handler_args
+            )
+
+    if LEVEL.get() == 1:
+        schedule_callback(0.1)
+
+
 def record_event(
     clock_id: int,
     event: str,
@@ -90,7 +156,7 @@ def analyze_stack(stack: List[Frame]):
         f'{CHAR_INFO} Finished transaction: {uuid.uuid4().hex}, '
         f'{total_time_seconds:.2f} seconds')
     print()
-    print('  #    Timestamp      %     Total    Nested Call Chain')
+    print('  #    Timestamp       %     Total    Nested Call Chain')
     print()
 
     counter: int = 0
@@ -111,22 +177,57 @@ def analyze_stack(stack: List[Frame]):
             raw_time_seconds: float = \
                 frame_childs[-1].timestamp - frame_childs[0].timestamp
 
-            raw_time_ratio: float = (
-                100.0 * raw_time_seconds / total_time_seconds
-                if total_time_seconds > 0.0
-                else 100.0
+            raw_time_ratio: float = 100.0 * divide(
+                numerator=raw_time_seconds,
+                denominator=total_time_seconds,
+                on_zero_denominator=1.0,
             )
 
             print(
                 f'{counter:>6} '
                 f'{relative_timestamp:>8.2f}s '
-                f'{raw_time_ratio:>5.1f}% '
+                f'{raw_time_ratio:>6.1f}% '
                 f'{raw_time_seconds:>8.2f}s '
                 f'{indentation} '
                 f'{frame.function}'
             )
 
-    print()
+    event_loop_timer_snapshots: List[EventLoopTimerSnapshot] = \
+        EVENT_LOOP_TIMER_SNAPSHOTS.get()
+
+    top_event_loop_timer_snapshots: List[EventLoopTimerSnapshot] = sorted(
+        [
+            snapshot
+            for snapshot in event_loop_timer_snapshots
+            if snapshot.was_blocked
+        ],
+        key=operator.attrgetter('real_tick_duration'),
+        reverse=True,
+    )
+
+    if top_event_loop_timer_snapshots:
+        print()
+        print(
+            f'  Some blocks (skews) occurred in the event loop'
+            f' {CHAR_SUPERSCRIPT_ONE}'
+        )
+        print()
+        print('  #    Timestamp  Excess     Delay')
+        print()
+        initial_timestamp: float = event_loop_timer_snapshots[0].timestamp
+        for counter, snapshot in enumerate(top_event_loop_timer_snapshots):
+            print(
+                f'{counter:>6} '
+                f'{snapshot.timestamp - initial_timestamp:>8.2f}s '
+                f'{100.0 * snapshot.block_duration_ratio:>6.1f}% '
+                f'{snapshot.real_tick_duration:>8.2f}s '
+            )
+        print()
+        print(
+            f'  {CHAR_SUPERSCRIPT_ONE}'
+            f' Consider reviewing it carefully'
+            f' to improve the overall system throughput'
+        )
 
 
 def _get_wrapper(  # noqa: MC001
@@ -151,15 +252,16 @@ def _get_wrapper(  # noqa: MC001
 
             @functools.wraps(function)
             async def wrapper(*args, **kwargs):
-                if not LEVEL.get():
+                if LEVEL.get() == 0:
                     STACK.set([])
 
                 with increase_counter(LEVEL):
+                    measure_event_loop_skew(clock_id)
                     record_event(clock_id, 'call', function, function_name)
                     result = await function(*args, **kwargs)
                     record_event(clock_id, 'return', function, function_name)
 
-                if not LEVEL.get():
+                if LEVEL.get() == 0:
                     stack_analyzer(STACK.get())
 
                 return result
@@ -168,7 +270,7 @@ def _get_wrapper(  # noqa: MC001
 
             @functools.wraps(function)
             def wrapper(*args, **kwargs):
-                if not LEVEL.get():
+                if LEVEL.get() == 0:
                     STACK.set([])
 
                 with increase_counter(LEVEL):
@@ -176,7 +278,7 @@ def _get_wrapper(  # noqa: MC001
                     result = function(*args, **kwargs)
                     record_event(clock_id, 'return', function, function_name)
 
-                if not LEVEL.get():
+                if LEVEL.get() == 0:
                     stack_analyzer(STACK.get())
 
                 return result
